@@ -126,16 +126,13 @@ class BaseAction(loadable.Loadable):
 
         :returns: A schema declaring the input parameters this action should be
                   provided along with their respective constraints
-        :rtype: :py:class:`voluptuous.Schema` instance
+        :rtype: :py:class:`jsonschema.Schema` instance
         """
         raise NotImplementedError()
 
     def validate_parameters(self):
-        try:
-            jsonschema.validate(self.input_parameters, self.schema)
-            return True
-        except jsonschema.ValidationError as e:
-            raise e
+        jsonschema.validate(self.input_parameters, self.schema)
+        return True
 
     @abc.abstractmethod
     def get_description(self):
