@@ -24,19 +24,17 @@ import string
 from croniter import croniter
 
 from jsonschema import validators
-from oslo_log import log as logging
+from oslo_config import cfg
+from oslo_log import log
 from oslo_utils import strutils
-from oslo_utils import timeutils
 from oslo_utils import uuidutils
 import six
 
 from watcher.common import exception
 
-from watcher import conf
+CONF = cfg.CONF
 
-CONF = conf.CONF
-
-LOG = logging.getLogger(__name__)
+LOG = log.getLogger(__name__)
 
 
 class Struct(dict):
@@ -65,7 +63,6 @@ class Struct(dict):
 generate_uuid = uuidutils.generate_uuid
 is_uuid_like = uuidutils.is_uuid_like
 is_int_like = strutils.is_int_like
-strtime = timeutils.strtime
 
 
 def is_cron_like(value):
