@@ -27,9 +27,12 @@ MONASCA_CLIENT_OPTS = [
                help='Version of Monasca API to use in monascaclient.'),
     cfg.StrOpt('interface',
                default='internal',
-               help='Type of interface used for monasca endpoint.'
-                    'Supported values: internal, public, admin'
-                    'The default is internal.')]
+               help='Type of interface used for monasca endpoint. '
+                    'Supported values: internal, public, admin. '
+                    'The default is internal.'),
+    cfg.StrOpt('region_name',
+               help='Region in Identity service catalog to use for '
+                    'communication with the OpenStack service.')]
 
 
 def register_opts(conf):
@@ -38,4 +41,4 @@ def register_opts(conf):
 
 
 def list_opts():
-    return [('monasca_client', MONASCA_CLIENT_OPTS)]
+    return [(monasca_client, MONASCA_CLIENT_OPTS)]
