@@ -16,6 +16,7 @@
 Openstack implementation of the cluster graph.
 """
 
+import ast
 from lxml import etree
 import networkx as nx
 from oslo_concurrency import lockutils
@@ -57,7 +58,7 @@ class ModelRoot(nx.DiGraph, base.Model):
     @lockutils.synchronized("model_root")
     def add_node(self, node):
         self.assert_node(node)
-        super(ModelRoot, self).add_node(node.uuid, node)
+        super(ModelRoot, self).add_node(node.uuid, attr=node)
 
     @lockutils.synchronized("model_root")
     def remove_node(self, node):
@@ -72,7 +73,7 @@ class ModelRoot(nx.DiGraph, base.Model):
     def add_instance(self, instance):
         self.assert_instance(instance)
         try:
-            super(ModelRoot, self).add_node(instance.uuid, instance)
+            super(ModelRoot, self).add_node(instance.uuid, attr=instance)
         except nx.NetworkXError as exc:
             LOG.exception(exc)
             raise exception.InstanceNotFound(name=instance.uuid)
@@ -86,10 +87,11 @@ class ModelRoot(nx.DiGraph, base.Model):
     def map_instance(self, instance, node):
         """Map a newly created instance to a node
 
-        :param instance: :py:class:`~.Instance` object or instance UUID
-        :type instance: str or :py:class:`~.Instance`
-        :param node: :py:class:`~.ComputeNode` object or node UUID
-        :type node: str or :py:class:`~.Instance`
+        :param instance: :py:class:`~.instance.Instance` object or instance
+           UUID
+        :type instance: str or :py:class:`~.instance.Instance`
+        :param node: :py:class:`~.node.ComputeNode` object or node UUID
+        :type node: str or :py:class:`~.instance.Instance`
         """
         if isinstance(instance, six.string_types):
             instance = self.get_instance_by_uuid(instance)
@@ -137,8 +139,8 @@ class ModelRoot(nx.DiGraph, base.Model):
 
     @lockutils.synchronized("model_root")
     def get_all_compute_nodes(self):
-        return {uuid: cn for uuid, cn in self.nodes(data=True)
-                if isinstance(cn, element.ComputeNode)}
+        return {uuid: cn['attr'] for uuid, cn in self.nodes(data=True)
+                if isinstance(cn['attr'], element.ComputeNode)}
 
     @lockutils.synchronized("model_root")
     def get_node_by_uuid(self, uuid):
@@ -147,6 +149,19 @@ class ModelRoot(nx.DiGraph, base.Model):
         except exception.ComputeResourceNotFound:
             raise exception.ComputeNodeNotFound(name=uuid)
 
+    @lockutils.synchronized("model_root")
+    def get_node_by_name(self, name):
+        try:
+            node_list = [cn['attr'] for uuid, cn in self.nodes(data=True)
+                         if (isinstance(cn['attr'], element.ComputeNode) and
+                         cn['attr']['hostname'] == name)]
+            if node_list:
+                return node_list[0]
+            else:
+                raise exception.ComputeResourceNotFound
+        except exception.ComputeResourceNotFound:
+            raise exception.ComputeNodeNotFound(name=name)
+
     @lockutils.synchronized("model_root")
     def get_instance_by_uuid(self, uuid):
         try:
@@ -156,7 +171,7 @@ class ModelRoot(nx.DiGraph, base.Model):
 
     def _get_by_uuid(self, uuid):
         try:
-            return self.node[uuid]
+            return self.node[uuid]['attr']
         except Exception as exc:
             LOG.exception(exc)
             raise exception.ComputeResourceNotFound(name=uuid)
@@ -168,12 +183,12 @@ class ModelRoot(nx.DiGraph, base.Model):
             node = self._get_by_uuid(node_uuid)
             if isinstance(node, element.ComputeNode):
                 return node
-        raise exception.ComputeNodeNotFound(name=instance_uuid)
+        raise exception.InstanceNotMapped(uuid=instance_uuid)
 
     @lockutils.synchronized("model_root")
     def get_all_instances(self):
-        return {uuid: inst for uuid, inst in self.nodes(data=True)
-                if isinstance(inst, element.Instance)}
+        return {uuid: inst['attr'] for uuid, inst in self.nodes(data=True)
+                if isinstance(inst['attr'], element.Instance)}
 
     @lockutils.synchronized("model_root")
     def get_node_instances(self, node):
@@ -186,6 +201,25 @@ class ModelRoot(nx.DiGraph, base.Model):
 
         return node_instances
 
+    def get_node_used_resources(self, node):
+        vcpu_used = 0
+        memory_used = 0
+        disk_used = 0
+        for instance in self.get_node_instances(node):
+            vcpu_used += instance.vcpus
+            memory_used += instance.memory
+            disk_used += instance.disk
+
+        return dict(vcpu=vcpu_used, memory=memory_used, disk=disk_used)
+
+    def get_node_free_resources(self, node):
+        resources_used = self.get_node_used_resources(node)
+        vcpu_free = node.vcpu_capacity-resources_used.get('vcpu')
+        memory_free = node.memory_mb_capacity-resources_used.get('memory')
+        disk_free = node.disk_gb_capacity-resources_used.get('disk')
+
+        return dict(vcpu=vcpu_free, memory=memory_free, disk=disk_free)
+
     def to_string(self):
         return self.to_xml()
 
@@ -209,11 +243,33 @@ class ModelRoot(nx.DiGraph, base.Model):
                                key=lambda inst: inst.uuid):
             try:
                 self.get_node_by_instance_uuid(instance.uuid)
-            except (exception.InstanceNotFound, exception.ComputeNodeNotFound):
+            except exception.ComputeResourceNotFound:
                 root.append(instance.as_xml_element())
 
         return etree.tostring(root, pretty_print=True).decode('utf-8')
 
+    def to_list(self):
+        ret_list = []
+        for cn in sorted(self.get_all_compute_nodes().values(),
+                         key=lambda cn: cn.uuid):
+            in_dict = {}
+            for field in cn.fields:
+                new_name = "node_"+str(field)
+                in_dict[new_name] = cn[field]
+            node_instances = self.get_node_instances(cn)
+            if not node_instances:
+                deep_in_dict = in_dict.copy()
+                ret_list.append(deep_in_dict)
+                continue
+            for instance in sorted(node_instances, key=lambda x: x.uuid):
+                for field in instance.fields:
+                    new_name = "server_"+str(field)
+                    in_dict[new_name] = instance[field]
+                if in_dict != {}:
+                    deep_in_dict = in_dict.copy()
+                    ret_list.append(deep_in_dict)
+        return ret_list
+
     @classmethod
     def from_xml(cls, data):
         model = cls()
@@ -225,6 +281,8 @@ class ModelRoot(nx.DiGraph, base.Model):
 
         for inst in root.findall('.//Instance'):
             instance = element.Instance(**inst.attrib)
+            instance.watcher_exclude = ast.literal_eval(
+                inst.attrib["watcher_exclude"])
             model.add_instance(instance)
 
             parent = inst.getparent()
@@ -239,7 +297,7 @@ class ModelRoot(nx.DiGraph, base.Model):
     @classmethod
     def is_isomorphic(cls, G1, G2):
         def node_match(node1, node2):
-            return node1.as_dict() == node2.as_dict()
+            return node1['attr'].as_dict() == node2['attr'].as_dict()
         return nx.algorithms.isomorphism.isomorph.is_isomorphic(
             G1, G2, node_match=node_match)
 
@@ -277,12 +335,12 @@ class StorageModelRoot(nx.DiGraph, base.Model):
     @lockutils.synchronized("storage_model")
     def add_node(self, node):
         self.assert_node(node)
-        super(StorageModelRoot, self).add_node(node.host, node)
+        super(StorageModelRoot, self).add_node(node.host, attr=node)
 
     @lockutils.synchronized("storage_model")
     def add_pool(self, pool):
         self.assert_pool(pool)
-        super(StorageModelRoot, self).add_node(pool.name, pool)
+        super(StorageModelRoot, self).add_node(pool.name, attr=pool)
 
     @lockutils.synchronized("storage_model")
     def remove_node(self, node):
@@ -306,8 +364,8 @@ class StorageModelRoot(nx.DiGraph, base.Model):
     def map_pool(self, pool, node):
         """Map a newly created pool to a node
 
-        :param pool: :py:class:`~.Pool` object or pool name
-        :param node: :py:class:`~.StorageNode` object or node host
+        :param pool: :py:class:`~.node.Pool` object or pool name
+        :param node: :py:class:`~.node.StorageNode` object or node host
         """
         if isinstance(pool, six.string_types):
             pool = self.get_pool_by_pool_name(pool)
@@ -322,8 +380,8 @@ class StorageModelRoot(nx.DiGraph, base.Model):
     def unmap_pool(self, pool, node):
         """Unmap a pool from a node
 
-        :param pool: :py:class:`~.Pool` object or pool name
-        :param node: :py:class:`~.StorageNode` object or node name
+        :param pool: :py:class:`~.node.Pool` object or pool name
+        :param node: :py:class:`~.node.StorageNode` object or node name
         """
         if isinstance(pool, six.string_types):
             pool = self.get_pool_by_pool_name(pool)
@@ -335,7 +393,7 @@ class StorageModelRoot(nx.DiGraph, base.Model):
     @lockutils.synchronized("storage_model")
     def add_volume(self, volume):
         self.assert_volume(volume)
-        super(StorageModelRoot, self).add_node(volume.uuid, volume)
+        super(StorageModelRoot, self).add_node(volume.uuid, attr=volume)
 
     @lockutils.synchronized("storage_model")
     def remove_volume(self, volume):
@@ -350,8 +408,8 @@ class StorageModelRoot(nx.DiGraph, base.Model):
     def map_volume(self, volume, pool):
         """Map a newly created volume to a pool
 
-        :param volume: :py:class:`~.Volume` object or volume UUID
-        :param pool: :py:class:`~.Pool` object or pool name
+        :param volume: :py:class:`~.volume.Volume` object or volume UUID
+        :param pool: :py:class:`~.node.Pool` object or pool name
         """
         if isinstance(volume, six.string_types):
             volume = self.get_volume_by_uuid(volume)
@@ -366,8 +424,8 @@ class StorageModelRoot(nx.DiGraph, base.Model):
     def unmap_volume(self, volume, pool):
         """Unmap a volume from a pool
 
-        :param volume: :py:class:`~.Volume` object or volume UUID
-        :param pool: :py:class:`~.Pool` object or pool name
+        :param volume: :py:class:`~.volume.Volume` object or volume UUID
+        :param pool: :py:class:`~.node.Pool` object or pool name
         """
         if isinstance(volume, six.string_types):
             volume = self.get_volume_by_uuid(volume)
@@ -382,15 +440,11 @@ class StorageModelRoot(nx.DiGraph, base.Model):
 
     @lockutils.synchronized("storage_model")
     def get_all_storage_nodes(self):
-        return {host: cn for host, cn in self.nodes(data=True)
-                if isinstance(cn, element.StorageNode)}
+        return {host: cn['attr'] for host, cn in self.nodes(data=True)
+                if isinstance(cn['attr'], element.StorageNode)}
 
     @lockutils.synchronized("storage_model")
     def get_node_by_name(self, name):
-        """Get a node by node name
-
-        :param node: :py:class:`~.StorageNode` object or node name
-        """
         try:
             return self._get_by_name(name.split("#")[0])
         except exception.StorageResourceNotFound:
@@ -412,14 +466,14 @@ class StorageModelRoot(nx.DiGraph, base.Model):
 
     def _get_by_uuid(self, uuid):
         try:
-            return self.node[uuid]
+            return self.node[uuid]['attr']
         except Exception as exc:
             LOG.exception(exc)
             raise exception.StorageResourceNotFound(name=uuid)
 
     def _get_by_name(self, name):
         try:
-            return self.node[name]
+            return self.node[name]['attr']
         except Exception as exc:
             LOG.exception(exc)
             raise exception.StorageResourceNotFound(name=name)
@@ -456,8 +510,8 @@ class StorageModelRoot(nx.DiGraph, base.Model):
 
     @lockutils.synchronized("storage_model")
     def get_all_volumes(self):
-        return {name: vol for name, vol in self.nodes(data=True)
-                if isinstance(vol, element.Volume)}
+        return {name: vol['attr'] for name, vol in self.nodes(data=True)
+                if isinstance(vol['attr'], element.Volume)}
 
     @lockutils.synchronized("storage_model")
     def get_pool_volumes(self, pool):
@@ -569,7 +623,7 @@ class BaremetalModelRoot(nx.DiGraph, base.Model):
     @lockutils.synchronized("baremetal_model")
     def add_node(self, node):
         self.assert_node(node)
-        super(BaremetalModelRoot, self).add_node(node.uuid, node)
+        super(BaremetalModelRoot, self).add_node(node.uuid, attr=node)
 
     @lockutils.synchronized("baremetal_model")
     def remove_node(self, node):
@@ -582,8 +636,8 @@ class BaremetalModelRoot(nx.DiGraph, base.Model):
 
     @lockutils.synchronized("baremetal_model")
     def get_all_ironic_nodes(self):
-        return {uuid: cn for uuid, cn in self.nodes(data=True)
-                if isinstance(cn, element.IronicNode)}
+        return {uuid: cn['attr'] for uuid, cn in self.nodes(data=True)
+                if isinstance(cn['attr'], element.IronicNode)}
 
     @lockutils.synchronized("baremetal_model")
     def get_node_by_uuid(self, uuid):
@@ -594,7 +648,7 @@ class BaremetalModelRoot(nx.DiGraph, base.Model):
 
     def _get_by_uuid(self, uuid):
         try:
-            return self.node[uuid]
+            return self.node[uuid]['attr']
         except Exception as exc:
             LOG.exception(exc)
         raise exception.BaremetalResourceNotFound(name=uuid)
