@@ -17,43 +17,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-"""
-*VM Workload Consolidation Strategy*
 
-A load consolidation strategy based on heuristic first-fit
-algorithm which focuses on measured CPU utilization and tries to
-minimize hosts which have too much or too little load respecting
-resource capacity constraints.
-
-This strategy produces a solution resulting in more efficient
-utilization of cluster resources using following four phases:
-
-* Offload phase - handling over-utilized resources
-* Consolidation phase - handling under-utilized resources
-* Solution optimization - reducing number of migrations
-* Disability of unused compute nodes
-
-A capacity coefficients (cc) might be used to adjust optimization
-thresholds. Different resources may require different coefficient
-values as well as setting up different coefficient values in both
-phases may lead to to more efficient consolidation in the end.
-If the cc equals 1 the full resource capacity may be used, cc
-values lower than 1 will lead to resource under utilization and
-values higher than 1 will lead to resource overbooking.
-e.g. If targeted utilization is 80 percent of a compute node capacity,
-the coefficient in the consolidation phase will be 0.8, but
-may any lower value in the offloading phase. The lower it gets
-the cluster will appear more released (distributed) for the
-following consolidation phase.
-
-As this strategy leverages VM live migration to move the load
-from one compute node to another, this feature needs to be set up
-correctly on all compute nodes within the cluster.
-This strategy assumes it is possible to live migrate any VM from
-an active compute node to any other active compute node.
-"""
-
-from oslo_config import cfg
 from oslo_log import log
 import six
 
@@ -66,37 +30,52 @@ LOG = log.getLogger(__name__)
 
 
 class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
-    """VM Workload Consolidation Strategy"""
+    """VM Workload Consolidation Strategy
 
-    HOST_CPU_USAGE_METRIC_NAME = 'compute.node.cpu.percent'
-    INSTANCE_CPU_USAGE_METRIC_NAME = 'cpu_util'
+    A load consolidation strategy based on heuristic first-fit
+    algorithm which focuses on measured CPU utilization and tries to
+    minimize hosts which have too much or too little load respecting
+    resource capacity constraints.
 
+    This strategy produces a solution resulting in more efficient
+    utilization of cluster resources using following four phases:
+
+    * Offload phase - handling over-utilized resources
+    * Consolidation phase - handling under-utilized resources
+    * Solution optimization - reducing number of migrations
+    * Disability of unused compute nodes
+
+    A capacity coefficients (cc) might be used to adjust optimization
+    thresholds. Different resources may require different coefficient
+    values as well as setting up different coefficient values in both
+    phases may lead to to more efficient consolidation in the end.
+    If the cc equals 1 the full resource capacity may be used, cc
+    values lower than 1 will lead to resource under utilization and
+    values higher than 1 will lead to resource overbooking.
+    e.g. If targeted utilization is 80 percent of a compute node capacity,
+    the coefficient in the consolidation phase will be 0.8, but
+    may any lower value in the offloading phase. The lower it gets
+    the cluster will appear more released (distributed) for the
+    following consolidation phase.
+
+    As this strategy leverages VM live migration to move the load
+    from one compute node to another, this feature needs to be set up
+    correctly on all compute nodes within the cluster.
+    This strategy assumes it is possible to live migrate any VM from
+    an active compute node to any other active compute node.
+    """
+
+    AGGREGATE = 'mean'
     DATASOURCE_METRICS = ['instance_ram_allocated', 'instance_cpu_usage',
                           'instance_ram_usage', 'instance_root_disk_size']
 
-    METRIC_NAMES = dict(
-        ceilometer=dict(
-            cpu_util_metric='cpu_util',
-            ram_util_metric='memory.resident',
-            ram_alloc_metric='memory',
-            disk_alloc_metric='disk.root.size'),
-        gnocchi=dict(
-            cpu_util_metric='cpu_util',
-            ram_util_metric='memory.resident',
-            ram_alloc_metric='memory',
-            disk_alloc_metric='disk.root.size'),
-    )
-
     MIGRATION = "migrate"
     CHANGE_NOVA_SERVICE_STATE = "change_nova_service_state"
 
     def __init__(self, config, osc=None):
         super(VMWorkloadConsolidation, self).__init__(config, osc)
-        self._ceilometer = None
-        self._gnocchi = None
         self.number_of_migrations = 0
         self.number_of_released_nodes = 0
-        # self.ceilometer_instance_data_cache = dict()
         self.datasource_instance_data_cache = dict()
 
     @classmethod
@@ -139,21 +118,12 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
             }
         }
 
-    @classmethod
-    def get_config_opts(cls):
-        return [
-            cfg.StrOpt(
-                "datasource",
-                help="Data source to use in order to query the needed metrics",
-                default="gnocchi",
-                choices=["ceilometer", "gnocchi"])
-        ]
-
     def get_available_compute_nodes(self):
         default_node_scope = [element.ServiceState.ENABLED.value,
                               element.ServiceState.DISABLED.value]
+        nodes = self.compute_model.get_all_compute_nodes().items()
         return {uuid: cn for uuid, cn in
-                self.compute_model.get_all_compute_nodes().items()
+                nodes
                 if cn.state == element.ServiceState.ONLINE.value and
                 cn.status in default_node_scope}
 
@@ -168,7 +138,7 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
             return instance.state.value
         else:
             LOG.error('Unexpected instance state type, '
-                      'state=%(state)s, state_type=%(st)s.' %
+                      'state=%(state)s, state_type=%(st)s.',
                       dict(state=instance.state,
                            st=type(instance.state)))
             raise exception.WatcherException
@@ -184,7 +154,7 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
             return node.status.value
         else:
             LOG.error('Unexpected node status type, '
-                      'status=%(status)s, status_type=%(st)s.' %
+                      'status=%(status)s, status_type=%(st)s.',
                       dict(status=node.status,
                            st=type(node.status)))
             raise exception.WatcherException
@@ -195,7 +165,8 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
         :param node: node object
         :return: None
         """
-        params = {'state': element.ServiceState.ENABLED.value}
+        params = {'state': element.ServiceState.ENABLED.value,
+                  'resource_name': node.hostname}
         self.solution.add_action(
             action_type=self.CHANGE_NOVA_SERVICE_STATE,
             resource_id=node.uuid,
@@ -209,7 +180,8 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
         :return: None
         """
         params = {'state': element.ServiceState.DISABLED.value,
-                  'disabled_reason': self.REASON_FOR_DISABLE}
+                  'disabled_reason': self.REASON_FOR_DISABLE,
+                  'resource_name': node.hostname}
         self.solution.add_action(
             action_type=self.CHANGE_NOVA_SERVICE_STATE,
             resource_id=node.uuid,
@@ -233,7 +205,7 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
             # migration mechanism to move non active VMs.
             LOG.error(
                 'Cannot live migrate: instance_uuid=%(instance_uuid)s, '
-                'state=%(instance_state)s.' % dict(
+                'state=%(instance_state)s.', dict(
                     instance_uuid=instance.uuid,
                     instance_state=instance_state_str))
             return
@@ -250,12 +222,11 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
 
         if self.compute_model.migrate_instance(
                 instance, source_node, destination_node):
-            params = {'migration_type': migration_type,
-                      'source_node': source_node.uuid,
-                      'destination_node': destination_node.uuid}
-            self.solution.add_action(action_type=self.MIGRATION,
-                                     resource_id=instance.uuid,
-                                     input_parameters=params)
+            self.add_action_migrate(
+                instance,
+                migration_type,
+                source_node,
+                destination_node)
             self.number_of_migrations += 1
 
     def disable_unused_nodes(self):
@@ -283,36 +254,21 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
         if instance.uuid in self.datasource_instance_data_cache.keys():
             return self.datasource_instance_data_cache.get(instance.uuid)
 
-        cpu_util_metric = self.METRIC_NAMES[
-            self.config.datasource]['cpu_util_metric']
-        ram_util_metric = self.METRIC_NAMES[
-            self.config.datasource]['ram_util_metric']
-        ram_alloc_metric = self.METRIC_NAMES[
-            self.config.datasource]['ram_alloc_metric']
-        disk_alloc_metric = self.METRIC_NAMES[
-            self.config.datasource]['disk_alloc_metric']
-
-        instance_cpu_util = self.datasource_backend.statistic_aggregation(
-            resource_id=instance.uuid,
-            meter_name=cpu_util_metric,
-            period=self.period,
-            granularity=self.granularity)
-        instance_ram_util = self.datasource_backend.statistic_aggregation(
-            resource_id=instance.uuid,
-            meter_name=ram_util_metric,
-            period=self.period,
-            granularity=self.granularity)
+        instance_cpu_util = self.datasource_backend.get_instance_cpu_usage(
+            resource=instance, period=self.period,
+            aggregate=self.AGGREGATE, granularity=self.granularity)
+        instance_ram_util = self.datasource_backend.get_instance_ram_usage(
+            resource=instance, period=self.period,
+            aggregate=self.AGGREGATE, granularity=self.granularity)
         if not instance_ram_util:
-            instance_ram_util = self.datasource_backend.statistic_aggregation(
-                resource_id=instance.uuid,
-                meter_name=ram_alloc_metric,
-                period=self.period,
-                granularity=self.granularity)
-        instance_disk_util = self.datasource_backend.statistic_aggregation(
-            resource_id=instance.uuid,
-            meter_name=disk_alloc_metric,
-            period=self.period,
-            granularity=self.granularity)
+            instance_ram_util = (
+                self.datasource_backend.get_instance_ram_allocated(
+                    resource=instance, period=self.period,
+                    aggregate=self.AGGREGATE, granularity=self.granularity))
+        instance_disk_util = (
+            self.datasource_backend.get_instance_root_disk_size(
+                resource=instance, period=self.period,
+                aggregate=self.AGGREGATE, granularity=self.granularity))
 
         if instance_cpu_util:
             total_cpu_utilization = (
@@ -362,7 +318,8 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
         :param node: node object
         :return: dict(cpu(cores), ram(MB), disk(B))
         """
-        return dict(cpu=node.vcpus, ram=node.memory, disk=node.disk_capacity)
+        return dict(cpu=node.vcpu_capacity, ram=node.memory_mb_capacity,
+                    disk=node.disk_gb_capacity)
 
     def get_relative_node_utilization(self, node):
         """Return relative node utilization.
@@ -463,13 +420,13 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
                     'input_parameters'][
                         'resource_id'] == instance_uuid)
             if len(actions) > 1:
-                src_uuid = actions[0]['input_parameters']['source_node']
-                dst_uuid = actions[-1]['input_parameters']['destination_node']
+                src_name = actions[0]['input_parameters']['source_node']
+                dst_name = actions[-1]['input_parameters']['destination_node']
                 for a in actions:
                     self.solution.actions.remove(a)
                     self.number_of_migrations -= 1
-                src_node = self.compute_model.get_node_by_uuid(src_uuid)
-                dst_node = self.compute_model.get_node_by_uuid(dst_uuid)
+                src_node = self.compute_model.get_node_by_name(src_name)
+                dst_node = self.compute_model.get_node_by_name(dst_name)
                 instance = self.compute_model.get_instance_by_uuid(
                     instance_uuid)
                 if self.compute_model.migrate_instance(
@@ -504,6 +461,11 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
                         key=lambda x: self.get_instance_utilization(
                             x)['cpu']
                 ):
+                    # skip exclude instance when migrating
+                    if instance.watcher_exclude:
+                        LOG.debug("Instance is excluded by scope, "
+                                  "skipped: %s", instance.uuid)
+                        continue
                     for destination_node in reversed(sorted_nodes):
                         if self.instance_fits(
                                 instance, destination_node, cc):
@@ -536,6 +498,11 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
                 self.compute_model.get_node_instances(node),
                 key=lambda x: self.get_instance_utilization(x)['cpu'])
             for instance in reversed(instances):
+                # skip exclude instance when migrating
+                if instance.watcher_exclude:
+                    LOG.debug("Instance is excluded by scope, "
+                              "skipped: %s", instance.uuid)
+                    continue
                 dsc = len(sorted_nodes) - 1
                 for destination_node in reversed(sorted_nodes):
                     if asc >= dsc:
@@ -549,15 +516,9 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
             asc += 1
 
     def pre_execute(self):
-        if not self.compute_model:
-            raise exception.ClusterStateNotDefined()
+        self._pre_execute()
 
-        if self.compute_model.stale:
-            raise exception.ClusterStateStale()
-
-        LOG.debug(self.compute_model.to_string())
-
-    def do_execute(self):
+    def do_execute(self, audit=None):
         """Execute strategy.
 
         This strategy produces a solution resulting in more
