@@ -19,11 +19,10 @@
 
 from __future__ import division
 
-from oslo_config import cfg
 from oslo_log import log
 
 from watcher._i18n import _
-from watcher.common import exception as wexc
+from watcher.common import exception
 from watcher.decision_engine.model import element
 from watcher.decision_engine.strategy.strategies import base
 
@@ -46,7 +45,7 @@ class WorkloadBalance(base.WorkloadStabilizationBaseStrategy):
         * Hardware: compute node should use the same physical CPUs/RAMs
         * Software: Ceilometer component ceilometer-agent-compute running
           in each compute node, and Ceilometer API can report such telemetry
-          "cpu_util" and "memory.resident" successfully.
+          "instance_cpu_usage" and "instance_ram_usage" successfully.
         * You must have at least 2 physical compute nodes to run this strategy.
 
     *Limitations*
@@ -60,16 +59,12 @@ class WorkloadBalance(base.WorkloadStabilizationBaseStrategy):
 
     # The meter to report CPU utilization % of VM in ceilometer
     # Unit: %, value range is [0 , 100]
-    CPU_METER_NAME = "cpu_util"
 
     # The meter to report memory resident of VM in ceilometer
     # Unit: MB
-    MEM_METER_NAME = "memory.resident"
 
     DATASOURCE_METRICS = ['instance_cpu_usage', 'instance_ram_usage']
 
-    MIGRATION = "migrate"
-
     def __init__(self, config, osc=None):
         """Workload balance using live migration
 
@@ -81,6 +76,7 @@ class WorkloadBalance(base.WorkloadStabilizationBaseStrategy):
         # the migration plan will be triggered when the CPU or RAM
         # utilization % reaches threshold
         self._meter = None
+        self.instance_migrations_count = 0
 
     @classmethod
     def get_name(cls):
@@ -107,8 +103,8 @@ class WorkloadBalance(base.WorkloadStabilizationBaseStrategy):
                     "description": "Workload balance based on metrics: "
                                    "cpu or ram utilization",
                     "type": "string",
-                    "choice": ["cpu_util", "memory.resident"],
-                    "default": "cpu_util"
+                    "choice": ["instance_cpu_usage", "instance_ram_usage"],
+                    "default": "instance_cpu_usage"
                 },
                 "threshold": {
                     "description": "workload threshold for migration",
@@ -129,19 +125,6 @@ class WorkloadBalance(base.WorkloadStabilizationBaseStrategy):
             },
         }
 
-    @classmethod
-    def get_config_opts(cls):
-        return [
-            cfg.ListOpt(
-                "datasources",
-                help="Datasources to use in order to query the needed metrics."
-                     " If one of strategy metric isn't available in the first"
-                     " datasource, the next datasource will be chosen.",
-                item_type=cfg.types.String(choices=['gnocchi', 'ceilometer',
-                                                    'monasca']),
-                default=['gnocchi', 'ceilometer', 'monasca'])
-        ]
-
     def get_available_compute_nodes(self):
         default_node_scope = [element.ServiceState.ENABLED.value]
         return {uuid: cn for uuid, cn in
@@ -170,7 +153,7 @@ class WorkloadBalance(base.WorkloadStabilizationBaseStrategy):
         :param workload_cache: the map contains instance to workload mapping
         """
         for instance_data in hosts:
-            source_node = instance_data['node']
+            source_node = instance_data['compute_node']
             source_instances = self.compute_model.get_node_instances(
                 source_node)
             if source_instances:
@@ -195,7 +178,7 @@ class WorkloadBalance(base.WorkloadStabilizationBaseStrategy):
                         if 0 <= current_delta < min_delta:
                             min_delta = current_delta
                             instance_id = instance.uuid
-                    except wexc.InstanceNotFound:
+                    except exception.InstanceNotFound:
                         LOG.error("Instance not found; error: %s",
                                   instance_id)
                 if instance_id:
@@ -203,7 +186,7 @@ class WorkloadBalance(base.WorkloadStabilizationBaseStrategy):
                             self.compute_model.get_instance_by_uuid(
                                 instance_id))
             else:
-                LOG.info("VM not found from node: %s",
+                LOG.info("VM not found from compute_node: %s",
                          source_node.uuid)
 
     def filter_destination_hosts(self, hosts, instance_to_migrate,
@@ -217,22 +200,22 @@ class WorkloadBalance(base.WorkloadStabilizationBaseStrategy):
         destination_hosts = []
         src_instance_workload = workload_cache[instance_to_migrate.uuid]
         for instance_data in hosts:
-            host = instance_data['node']
+            host = instance_data['compute_node']
             workload = instance_data['workload']
             # calculate the available resources
             cores_used, mem_used, disk_used = self.calculate_used_resource(
                 host)
-            cores_available = host.vcpus - cores_used
-            disk_available = host.disk - disk_used
-            mem_available = host.memory - mem_used
+            cores_available = host.vcpu_capacity - cores_used
+            disk_available = host.disk_gb_capacity - disk_used
+            mem_available = host.memory_mb_capacity - mem_used
             if (cores_available >= required_cores and
                     mem_available >= required_mem and
                     disk_available >= required_disk):
-                if (self._meter == self.CPU_METER_NAME and
+                if (self._meter == 'instance_cpu_usage' and
                     ((src_instance_workload + workload) <
                      self.threshold / 100 * host.vcpus)):
                     destination_hosts.append(instance_data)
-                if (self._meter == self.MEM_METER_NAME and
+                if (self._meter == 'instance_ram_usage' and
                     ((src_instance_workload + workload) <
                      self.threshold / 100 * host.memory)):
                     destination_hosts.append(instance_data)
@@ -240,7 +223,7 @@ class WorkloadBalance(base.WorkloadStabilizationBaseStrategy):
         return destination_hosts
 
     def group_hosts_by_cpu_or_ram_util(self):
-        """Calculate the workloads of each node
+        """Calculate the workloads of each compute_node
 
         try to find out the nodes which have reached threshold
         and the nodes which are under threshold.
@@ -250,8 +233,6 @@ class WorkloadBalance(base.WorkloadStabilizationBaseStrategy):
 
         nodes = self.get_available_compute_nodes()
         cluster_size = len(nodes)
-        if not nodes:
-            raise wexc.ClusterEmpty()
         overload_hosts = []
         nonoverload_hosts = []
         # total workload of cluster
@@ -266,19 +247,18 @@ class WorkloadBalance(base.WorkloadStabilizationBaseStrategy):
                 util = None
                 try:
                     util = self.datasource_backend.statistic_aggregation(
-                        instance.uuid, self._meter, self._period,
-                        self._granularity, aggregation='mean',
-                        dimensions=dict(resource_id=instance.uuid))
+                        instance, 'instance', self._meter, self._period,
+                        'mean', self._granularity)
                 except Exception as exc:
                     LOG.exception(exc)
                     LOG.error("Can not get %s from %s", self._meter,
-                              self.config.datasource)
+                              self.datasource_backend.NAME)
                     continue
                 if util is None:
                     LOG.debug("Instance (%s): %s is None",
                               instance.uuid, self._meter)
                     continue
-                if self._meter == self.CPU_METER_NAME:
+                if self._meter == 'instance_cpu_usage':
                     workload_cache[instance.uuid] = (util *
                                                      instance.vcpus / 100)
                 else:
@@ -288,13 +268,13 @@ class WorkloadBalance(base.WorkloadStabilizationBaseStrategy):
                           util)
 
             cluster_workload += node_workload
-            if self._meter == self.CPU_METER_NAME:
+            if self._meter == 'instance_cpu_usage':
                 node_util = node_workload / node.vcpus * 100
             else:
                 node_util = node_workload / node.memory * 100
 
             instance_data = {
-                'node': node, self._meter: node_util,
+                'compute_node': node, self._meter: node_util,
                 'workload': node_workload}
             if node_util >= self.threshold:
                 # mark the node to release resources
@@ -302,34 +282,24 @@ class WorkloadBalance(base.WorkloadStabilizationBaseStrategy):
             else:
                 nonoverload_hosts.append(instance_data)
 
-        avg_workload = cluster_workload / cluster_size
+        avg_workload = 0
+        if cluster_size != 0:
+            avg_workload = cluster_workload / cluster_size
 
         return overload_hosts, nonoverload_hosts, avg_workload, workload_cache
 
     def pre_execute(self):
-        """Pre-execution phase
-
-        This can be used to fetch some pre-requisites or data.
-        """
-        LOG.info("Initializing Workload Balance Strategy")
-
-        if not self.compute_model:
-            raise wexc.ClusterStateNotDefined()
-
-        if self.compute_model.stale:
-            raise wexc.ClusterStateStale()
-
-        LOG.debug(self.compute_model.to_string())
-
-    def do_execute(self):
-        """Strategy execution phase
-
-        This phase is where you should put the main logic of your strategy.
-        """
+        self._pre_execute()
         self.threshold = self.input_parameters.threshold
         self._period = self.input_parameters.period
         self._meter = self.input_parameters.metrics
         self._granularity = self.input_parameters.granularity
+
+    def do_execute(self, audit=None):
+        """Strategy execution phase
+
+        This phase is where you should put the main logic of your strategy.
+        """
         source_nodes, target_nodes, avg_workload, workload_cache = (
             self.group_hosts_by_cpu_or_ram_util())
 
@@ -367,16 +337,16 @@ class WorkloadBalance(base.WorkloadStabilizationBaseStrategy):
         destination_hosts = sorted(destination_hosts,
                                    key=lambda x: (x[self._meter]))
         # always use the host with lowerest CPU utilization
-        mig_destination_node = destination_hosts[0]['node']
+        mig_destination_node = destination_hosts[0]['compute_node']
         # generate solution to migrate the instance to the dest server,
         if self.compute_model.migrate_instance(
                 instance_src, source_node, mig_destination_node):
-            parameters = {'migration_type': 'live',
-                          'source_node': source_node.uuid,
-                          'destination_node': mig_destination_node.uuid}
-            self.solution.add_action(action_type=self.MIGRATION,
-                                     resource_id=instance_src.uuid,
-                                     input_parameters=parameters)
+            self.add_action_migrate(
+                instance_src,
+                'live',
+                source_node,
+                mig_destination_node)
+            self.instance_migrations_count += 1
 
     def post_execute(self):
         """Post-execution phase
@@ -384,5 +354,9 @@ class WorkloadBalance(base.WorkloadStabilizationBaseStrategy):
         This can be used to compute the global efficacy
         """
         self.solution.model = self.compute_model
+        self.solution.set_efficacy_indicators(
+            instance_migrations_count=self.instance_migrations_count,
+            instances_count=len(self.compute_model.get_all_instances())
+        )
 
         LOG.debug(self.compute_model.to_string())
