@@ -79,7 +79,6 @@ class FunctionalTest(base.DbTestCase):
                     hooks.ContextHook(),
                     hooks.NoExceptionTracebackHook()
                 ],
-                'static_root': '%s/public' % root_dir,
                 'template_path': '%s/api/templates' % root_dir,
                 'enable_acl': enable_acl,
                 'acl_public_routes': ['/', '/v1'],
@@ -137,6 +136,12 @@ class FunctionalTest(base.DbTestCase):
                                   headers=headers, extra_environ=extra_environ,
                                   status=status, method="put")
 
+    def post(self, *args, **kwargs):
+        headers = kwargs.pop('headers', {})
+        headers.setdefault('Accept', 'application/json')
+        kwargs['headers'] = headers
+        return self.app.post(*args, **kwargs)
+
     def post_json(self, path, params, expect_errors=False, headers=None,
                   extra_environ=None, status=None):
         """Sends simulated HTTP POST request to Pecan test app.
@@ -197,7 +202,8 @@ class FunctionalTest(base.DbTestCase):
         return response
 
     def get_json(self, path, expect_errors=False, headers=None,
-                 extra_environ=None, q=[], path_prefix=PATH_PREFIX, **params):
+                 extra_environ=None, q=[], path_prefix=PATH_PREFIX,
+                 return_json=True, **params):
         """Sends simulated HTTP GET request to Pecan test app.
 
         :param path: url path of target service
@@ -230,7 +236,7 @@ class FunctionalTest(base.DbTestCase):
                                 headers=headers,
                                 extra_environ=extra_environ,
                                 expect_errors=expect_errors)
-        if not expect_errors:
+        if return_json and not expect_errors:
             response = response.json
         print('GOT:%s' % response)
         return response
